@@ -1,6 +1,7 @@
 package Net::OSCAR::_BLInternal;
 
 use Net::OSCAR::Common qw(:all);
+use Net::OSCAR::OldPerl;
 
 # Heh, this is fun.
 # This is what we use as the first arg to Net::OSCAR::TLV when creating a new BLI.
@@ -43,7 +44,7 @@ sub blparse($$) {
 	while(length($data) > 4) {
 		my($name) = unpack("n/a*", $data);
 		substr($data, 0, 2+length($name)) = "";
-		my($gid, $bid, $type, $sublen) = unpack("n*", substr($data, 0, 8, ""));
+		my($gid, $bid, $type, $sublen) = unpack("n4", substr($data, 0, 8, ""));
 		my $typedata = tlv_decode(substr($data, 0, $sublen, ""));
 
 		$session->{blinternal}->{$type}->{$gid}->{$bid}->{name} = $name if $name;
@@ -59,82 +60,37 @@ sub blparse($$) {
 # Buddylist-Internal -> Net::OSCAR
 # Sets various $session hashkeys from blinternal.
 # That's what Brian Bli-to-no'd do. ;)
-#
-# Optionally, only process one type/GID/BID.
-# All three must be specified if any are.
-# This is used by Callbacks.pm when we get an 0x13/0x0E error.
-sub BLI_to_NO($;$$$) {
-	my($session, $dotype, $dogid, $dobid) = @_;
+sub BLI_to_NO($) {
+	my($session) = @_;
 	my $bli = $session->{blinternal};
 
-	if(!defined($dotype)) {
-		delete $session->{buddies};
-		delete $session->{permit};
-		delete $session->{deny};
-		delete $session->{visibility};
-		delete $session->{groupperms};
-		delete $session->{profile};
-		delete $session->{appdata};
-		delete $session->{showidle};
+	delete $session->{buddies};
+	delete $session->{permit};
+	delete $session->{deny};
+	delete $session->{visibility};
+	delete $session->{groupperms};
+	delete $session->{profile};
+	delete $session->{appdata};
+	delete $session->{showidle};
 
-		$session->{buddies} = bltie(1);
-		$session->{permit} = bltie;
-		$session->{deny} = bltie;
-	} else {
-		if($dotype == 0) {
-			my $group = $session->findgroup($dogid);
-			if($group and !exists($session->{blold}->{0}->{$dogid})) {
-				delete $bli->{0}->{$dogid};
-				delete $session->{buddies}->{$group};
-				return 1;
-			}
-		} elsif($dotype == 1) {
-			my $group = $session->findgroup($dogid);
-			if($group) {
-				my $buddy = $session->findbuddy_byid($session->{buddies}->{$group}->{members}, $dobid);
-				if($buddy and !exists($session->{blold}->{1}->{$dogid}->{$dobid})) {
-					delete $bli->{1}->{$dogid}->{$dobid};
-					delete $session->{buddies}->{$group}->{members}->{$buddy};
-					return 1;
-				}
-			}
-		} elsif($dotype == 2) {
-				my $buddy = $session->findbuddy_byid($session->{permit}, $dobid);
-				if($buddy and !exists($session->{blold}->{2}->{$dogid}->{$dobid})) {
-					delete $bli->{2}->{$dogid}->{$dobid};
-					delete $session->{permit}->{$buddy};
-					return 1;
-				}
-		} elsif($dotype == 3) {
-				my $buddy = $session->findbuddy_byid($session->{deny}, $dobid);
-				if($buddy and !exists($session->{blold}->{3}->{$dogid}->{$dobid})) {
-					delete $bli->{3}->{$dogid}->{$dobid};
-					delete $session->{deny}->{$buddy};
-					return 1;
-				}
-		} elsif($dotype == 4) {
-			delete $session->{visibility};
-			delete $session->{groupperms};
-			delete $session->{profile};
-			delete $session->{appdata};
-		} elsif($dotype == 5) {
-			delete $session->{showidle};
-		}
-	}
+	$session->{buddies} = bltie(1);
+	$session->{permit} = bltie;
+	$session->{deny} = bltie;
 
-	if(exists $bli->{2} and (!defined($dotype) or $dotype == 2)) {
-		foreach my $bid(defined($dobid) ? ($dobid) : keys(%{$bli->{2}->{0}})) {
+
+	if(exists $bli->{2}) {
+		foreach my $bid(keys(%{$bli->{2}->{0}})) {
 			$session->{permit}->{$bli->{2}->{0}->{$bid}->{name}} = {buddyid => $bid};
 		}
 	}
 
-	if(exists $bli->{3} and (!defined($dotype) or $dotype == 3)) {
-		foreach my $bid(defined($dobid) ? ($dobid) : keys(%{$bli->{3}->{0}})) {
+	if(exists $bli->{3}) {
+		foreach my $bid(keys(%{$bli->{3}->{0}})) {
 			$session->{deny}->{$bli->{3}->{0}->{$bid}->{name}} = {buddyid => $bid};
 		}
 	}
 
-	if(exists $bli->{4} and (!defined($dotype) or $dotype == 4)) {
+	if(exists $bli->{4}) {
 		my $typedata = $bli->{4}->{0}->{(keys %{$bli->{4}->{0}})[0]}->{data};
 		($session->{visibility}) = unpack("C", $typedata->{0xCA}) if $typedata->{0xCA};
 
@@ -150,7 +106,7 @@ sub BLI_to_NO($;$$$) {
 		$session->set_info($session->{profile}) if exists($session->{profile});
 	}
 
-	if(exists $bli->{5} and (!defined($dotype) or $dotype == 5)) {
+	if(exists $bli->{5}) {
 		# Not yet implemented
 		($session->{showidle}) = unpack("N", $bli->{5}->{0}->{19719}->{data}->{0xC9});
 	}
@@ -165,21 +121,6 @@ sub BLI_to_NO($;$$$) {
 		$_ != 0
 	} keys %{exists($bli->{1}) ? $bli->{1} : {}}; # That we have a type 1 entry for
 
-	if(defined($dotype)) {
-		if($dotype == 0 and $dogid == 0) { # Reset group order
-			tied(%{$session->{buddies}})->setorder(
-				map {
-					$session->findgroup($_)
-				} @gids
-			);
-			@gids = ();
-		} elsif($dotype == 1 or $dotype == 0) { # We're doing a group/buddy
-			@bids = ($dobid);
-		} else { # We're doing something else
-			@bids = ();
-		}
-	}
-
 	foreach my $gid(@gids) {
 		my $group = $bli->{1}->{$gid}->{0}->{name};
 
@@ -190,11 +131,9 @@ sub BLI_to_NO($;$$$) {
 		$session->{buddies}->{$group} ||= {};
 		my $entry = $session->{buddies}->{$group};
 
-		if(!defined($dotype) or $dotype == 1) { #Don't do this if doing a bud
-			$entry->{groupid} = $gid;
-			$entry->{members} = bltie unless $entry->{members};
-			$entry->{data} = $bli->{1}->{$gid}->{0}->{data};
-		}
+		$entry->{groupid} = $gid;
+		$entry->{members} = bltie unless $entry->{members};
+		$entry->{data} = $bli->{1}->{$gid}->{0}->{data};
 
 		my @bids = unpack("n*", $bli->{1}->{$gid}->{0}->{data}->{0xC8} || "");
 		delete $bli->{1}->{$gid}->{0}->{data}->{0xC8};
@@ -206,22 +145,10 @@ sub BLI_to_NO($;$$$) {
 			} @bids
 		} keys %{exists($bli->{0}->{$gid}) ? $bli->{0}->{$gid} : {}}; # That we have a type 0 entry for in this GID
 
-		if(defined($dotype)) {
-			if($dotype == 1) { # Reset buddy order
-				tied(%{$entry->{members}})->setorder(
-					map {
-						$session->findbuddy_byid($entry->{members}, $_)
-					} @bids
-				);
-				@bids = ();
-			} elsif($dotype == 0) { # We're doing a buddy
-				@bids = ($dobid);
-			} else { # We shouldn't even be here in this case, actually
-				@bids = ();
-			}
-		}
-
 		foreach my $bid(@bids) {
+			# Yeah, this next condition seems impossible, but I've seen it happen
+			next unless exists($bli->{0}->{$gid}) and exists($bli->{0}->{$gid}->{$bid});
+
 			my $buddy = $bli->{0}->{$gid}->{$bid};
 
 			my $comment = undef;
@@ -250,7 +177,7 @@ sub NO_to_BLI($) {
 	}
 
 	foreach my $deny (keys %{$session->{deny}}) {
-		$bli->{2}->{0}->{$session->{deny}->{$deny}->{buddyid}}->{name} = $deny;
+		$bli->{3}->{0}->{$session->{deny}->{$deny}->{buddyid}}->{name} = $deny;
 	}
 
 	my $vistype;
@@ -270,10 +197,14 @@ sub NO_to_BLI($) {
 	$bli->{1}->{0}->{0}->{data}->{0xC8} = pack("n*", map { $_->{groupid} } values %{$session->{buddies}});
 	foreach my $group(keys %{$session->{buddies}}) {
 		my $gid = $session->{buddies}->{$group}->{groupid};
-		$bli->{1}->{$gid}->{0}->{data}->{0xC8} = pack("n*", map { $_->{buddyid} } values %{$session->{buddies}->{$group}->{members}});
+		$bli->{1}->{$gid}->{0}->{name} = $group;
+		$bli->{1}->{$gid}->{0}->{data}->{0xC8} = pack("n*",
+			map { $_->{buddyid} }
+			values %{$session->{buddies}->{$group}->{members}});
 
 		foreach my $buddy(keys %{$session->{buddies}->{$group}->{members}}) {
 			my $bid = $session->{buddies}->{$group}->{members}->{$buddy}->{buddyid};
+			next unless $bid;
 			$bli->{0}->{$gid}->{$bid}->{name} = $buddy;
 			while(my ($key, $value) = each(%{$session->{buddies}->{$group}->{members}->{$buddy}->{data}})) {
 				$bli->{0}->{$gid}->{$bid}->{data}->{$key} = $value;
@@ -328,7 +259,7 @@ sub BLI_to_OSCAR($$) {
 					$session->log_print(OSCAR_DBG_DEBUG, "Deleting.");
 					$modcount++;
 
-					$oscar->snac_put(family => 0x13, subtype => 0xA, reqdata => {desc => "deleting ".(BUDTYPES)[$type]." $newentry->{name}", type => $type, gid => $gid, bid => $bid}, data => 
+					$oscar->snac_put(family => 0x13, subtype => 0xA, reqdata => {desc => "deleting ".(BUDTYPES)[$type]." $oldentry->{name}", type => $type, gid => $gid, bid => $bid}, data => 
 						pack("nnnnn", 0, $gid, $bid, $type, 0)
 					);
 				}
